@@ -13,6 +13,7 @@ from tests import get_testdata
 class Base:
     class LinkExtractorTestCase(unittest.TestCase):
         extractor_cls = None
+        escapes_whitespace = False
 
         def setUp(self):
             body = get_testdata('link_extractor', 'sgml_linkextractor.html')
@@ -26,12 +27,19 @@ class Base:
 
         def test_extract_all_links(self):
             lx = self.extractor_cls()
+            if self.escapes_whitespace:
+                page4_url = 'http://example.com/page%204.html'
+            else:
+                page4_url = 'http://example.com/page 4.html'
+
             self.assertEqual([link for link in lx.extract_links(self.response)], [
                 Link(url='http://example.com/sample1.html', text=u''),
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html#foo', text='sample 3 repetition with fragment'),
                 Link(url='http://www.google.com/something', text=u''),
                 Link(url='http://example.com/innertag.html', text=u'inner tag'),
+                Link(url=page4_url, text=u'href with whitespaces'),
             ])
 
         def test_extract_filter_allow(self):
@@ -40,6 +48,7 @@ class Base:
                 Link(url='http://example.com/sample1.html', text=u''),
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html#foo', text='sample 3 repetition with fragment')
             ])
 
         def test_extract_filter_allow_with_duplicates(self):
@@ -49,6 +58,27 @@ class Base:
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 repetition'),
+                Link(url='http://example.com/sample3.html#foo', text='sample 3 repetition with fragment')
+            ])
+
+        def test_extract_filter_allow_with_duplicates_canonicalize(self):
+            lx = self.extractor_cls(allow=('sample', ), unique=False,
+                                    canonicalize=True)
+            self.assertEqual([link for link in lx.extract_links(self.response)], [
+                Link(url='http://example.com/sample1.html', text=u''),
+                Link(url='http://example.com/sample2.html', text=u'sample 2'),
+                Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html', text=u'sample 3 repetition'),
+                Link(url='http://example.com/sample3.html', text='sample 3 repetition with fragment')
+            ])
+
+        def test_extract_filter_allow_no_duplicates_canonicalize(self):
+            lx = self.extractor_cls(allow=('sample',), unique=True,
+                                    canonicalize=True)
+            self.assertEqual([link for link in lx.extract_links(self.response)], [
+                Link(url='http://example.com/sample1.html', text=u''),
+                Link(url='http://example.com/sample2.html', text=u'sample 2'),
+                Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
             ])
 
         def test_extract_filter_allow_and_deny(self):
@@ -72,6 +102,8 @@ class Base:
                 Link(url='http://example.com/sample1.html', text=u''),
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html#foo',
+                     text='sample 3 repetition with fragment')
             ])
 
             lx = self.extractor_cls(allow='sample', deny='3')
@@ -275,12 +307,19 @@ class Base:
 
         def test_attrs(self):
             lx = self.extractor_cls(attrs="href")
+            if self.escapes_whitespace:
+                page4_url = 'http://example.com/page%204.html'
+            else:
+                page4_url = 'http://example.com/page 4.html'
+
             self.assertEqual(lx.extract_links(self.response), [
                 Link(url='http://example.com/sample1.html', text=u''),
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html#foo', text='sample 3 repetition with fragment'),
                 Link(url='http://www.google.com/something', text=u''),
                 Link(url='http://example.com/innertag.html', text=u'inner tag'),
+                Link(url=page4_url, text=u'href with whitespaces'),
             ])
 
             lx = self.extractor_cls(attrs=("href","src"), tags=("a","area","img"), deny_extensions=())
@@ -289,8 +328,10 @@ class Base:
                 Link(url='http://example.com/sample2.html', text=u'sample 2'),
                 Link(url='http://example.com/sample2.jpg', text=u''),
                 Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+                Link(url='http://example.com/sample3.html#foo', text='sample 3 repetition with fragment'),
                 Link(url='http://www.google.com/something', text=u''),
                 Link(url='http://example.com/innertag.html', text=u'inner tag'),
+                Link(url=page4_url, text=u'href with whitespaces'),
             ])
 
             lx = self.extractor_cls(attrs=None)
